<?php

namespace Drupal\config_enforce\Form;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Attached Config Enforce behaviour for config forms.
 */
abstract class AbstractConfigEnforceForm extends ConfigFormBase {

  use StringTranslationTrait;

  /* The form array. */
  protected $form;

  /* The form state. */
  protected $formState;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return get_called_class()::FORM_ID;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      ConfigEnforce::CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setSharedObjectProperties($form, $form_state);
  }

  /**
   * Set common properties on the object, for easier OOP.
   */
  protected function setSharedObjectProperties(array &$form, FormStateInterface &$form_state) {
    $this->form = $form;
    $this->formState = $form_state;
    // @see: AbstractConfigEnforceFormHandler::addEnforceForm()
    $this->context = $this->formState->getBuildInfo()['args'];
  }

  /**
   * Make our form float above the config form to which it's attached.
   */
  protected function makeFormFloat() {
    $this->form['#prefix'] = "<div class='config-enforce-form'>";
    $this->form['#suffix'] = "</div>";
    $this->form['#attached']['library'][] = 'config_enforce/config-enforce';
  }

  /**
   * Add a details section with information about the config form.
   */
  protected function addConfigFormInfo() {
    $form_id = $this->getContext('form_id');

    $this->form['config_enforce'] = [
      '#type' => 'details',
      '#tree' =>TRUE,
      '#title' => $this->t('Config form: %id', ['%id' => $form_id]),
      '#open' => TRUE,
    ];

    $this->addEnforcedConfigItems();
  }

  protected function addEnforcedConfigItems(){
    foreach ($this->getFormConfigs() as $config) {
      $this->addEnforcedConfigItem($config);
    }
  }

  protected function addEnforcedConfigItem($config) {

    $this->form['config_enforce'][$config] = [
      '#type' => 'details',
      '#title' => $this->t('Config object: @config', ['@config' => $config]),
      '#open' => TRUE,
    ];

    $config_enforce_enabled = ConfigEnforce::isEnforced($config);
    $this->form['config_enforce'][$config]['config_enforce_enabled'] = [
      '#type' => 'item',
      '#title' => $this->t('Config enforce'),
      '#markup' => $config_enforce_enabled ? 'Enabled' : 'Disabled',
    ];

    if ($config_enforce_enabled) {

      $this->form['config_enforce'][$config]['config_enforce_module'] = [
        '#type' => 'item',
        '#title' => $this->t('Module'),
        '#markup' => ConfigEnforce::getModule($config),
      ];

      $this->form['config_enforce'][$config]['config_enforce_directory'] = [
        '#type' => 'item',
        '#title' => $this->t('Directory'),
        '#markup' => ConfigEnforce::getDirectory($config),
      ];

      $this->form['config_enforce'][$config]['config_enforce_path'] = [
        '#type' => 'item',
        '#title' => $this->t('File path'),
        '#markup' => ConfigEnforce::getPath($config),
      ];

      $this->form['config_enforce'][$config]['config_enforce_level'] = [
        '#type' => 'item',
        '#title' => $this->t('Enforcement level'),
        '#markup' => ConfigEnforce::getLevelLabel($config),
      ];

    }
  }

  /**
   * Add a warning to the top of this form.
   */
  protected function addWarning($message, $tokens) {
    $warning = $this->t($message, $tokens);
    $this->form['warning'] = [
      '#type' => 'markup',
      '#markup' => Markup::create("
        <div class='messages messages--warning'>
          {$warning}
        </div>
      "),
    ];
    $this->form['#attached']['library'][] = 'classy/messages';
  }

  /**
   * Return an item passed into the form from the config form to which we are attached.
   */
  protected function getContext($key = '') {
    $context = $this->formState->getBuildInfo()['args'];
    return empty($key) ? $context : $context[$key];
  }

  /**
   * Return a list of configs embedded in the current form object.
   */
  protected function getFormConfigs() {
    return $this->getContext('configs');
  }

  /**
   * Print common debug data.
   */
  protected function debug() {
    if (function_exists('dpm')) {
      dpm($this->form);
      dpm($this->formState);
      dpm($this->getContext());
      dpm($this->getFormConfigs());
    }
  }

}
